PROGRAM GAME2;
(*--------------------------------------------------------------------------*)
(* Lsning til GAME, der benytter MIN-MAX metoden og dynamisk programmering *)
(* Dette giver modsat GAME1 altid den optimale score.                       *)
(*--------------------------------------------------------------------------*)

USES PLAY;

CONST                                 (*------------------------------------*)
  ML = 100;                           (* Maksimale antal tal p pladen      *)
TYPE                                  (*------------------------------------*)
  Table = Array[1..ML,1..ML] OF WORD; (* Datatype for tabel, der husker n  *)
                                      (* gang beregnede max-scorer          *)
VAR                                   (*------------------------------------*)
  Move     : CHAR;                    (* 'L'/'R' - angiver side, der vlges *)
  Board    : Array[1..ML] OF BYTE;    (* Spilleplade, med op til ML tal     *)
  MaxTable : Table;                   (* Tabel, med max-scorer              *)
  Left     ,                          (* Aktuelle venstrekant               *)
  Right    : BYTE;                    (* Aktuelle hjrekant                 *)
  MyScore  ,                          (* Summen af mine tal                 *)
  YourScore: WORD;                    (* Summen af modstanderens tal        *)
  InFile   ,                          (* Input fil                          *)
  OutFile  : TEXT;                    (* Output fil                         *)
                                      (*------------------------------------*)

FUNCTION Min(v1,v2: WORD): WORD;
BEGIN
   IF v1<v2 THEN Min:=v1 ELSE Min:=V2;
END;

FUNCTION Max(v1,v2: WORD): WORD;
BEGIN
   IF v1>v2 THEN Max:=v1 ELSE Max:=V2;
END;

FUNCTION MaxScore(Starting, Ending: BYTE): WORD; FORWARD;

FUNCTION LeftScore(Starting, Ending: BYTE): WORD;
(* Beregner den bedste score, der kan opns ved at tage venstre tal. *)
(* Da modstanderen spiller optimalt, vil det vre MINIMUM af de to    *)
(* scorer, der fremkommer, nr modstander vlger venstre eller hjre. *)
VAR
  LeftValue: WORD;
BEGIN
   LeftValue:=Board[Starting];
   IF (Ending-Starting>1) THEN
      INC(LeftValue,MIN(MaxScore(Starting+2,Ending),MaxScore(Starting+1,Ending-1)));
   LeftScore:=LeftValue;
END;

FUNCTION RightScore(Starting, Ending: BYTE): WORD;
(* Beregner den bedste score, der kan opns ved at tage hjre tal.   *)
(* Da modstanderen spiller optimalt, vil det vre MINIMUM af de to    *)
(* scorer, der fremkommer, nr modstander vlger venstre eller hjre. *)
VAR
  RightValue: WORD;
BEGIN
   RightValue:=Board[Ending];
   IF (Ending-Starting>1) THEN
      INC(RightValue,MIN(MaxScore(Starting, Ending-2),MaxScore(Starting+1,Ending-1)));
   RightScore:=RightValue;
END;

FUNCTION MaxScore(Starting, Ending: BYTE): WORD;
(* Beregner den maksimale score ved at tage maksimum af den bedste *)
(* af de to scorer, der fremkommer ved at vlge venstre hhv. hjre *)
(* tal.  Af tids- og plads- (stack) mssige rsager gemmes Max-    *)
(* scoren i en tabel (dynamisk programmering), s allerede besgte *)
(* konfigurationer ikke giver anledning til ny rekursion.          *)
BEGIN
   IF MaxTable[Starting, Ending]=0 THEN  (* Endnu ikke udregnet *)
      MaxTable[Starting, Ending]:=Max(LeftScore(Starting, Ending),RightScore(Starting, Ending));
   MaxScore:=MaxTable[Starting, Ending];
END;

PROCEDURE Init;
BEGIN
   (* Initialise table and variables *)
      FOR Left:=1 TO ML DO
        FOR Right:=1 TO ML DO MaxTable[Left,Right]:=0;
      MyScore:=0; YourScore:=0;

   (* Read input *)
      ASSIGN(InFile,Inname); RESET(InFile);
      ReadLn(InFile,Right);
      FOR Left:=1 TO Right DO ReadLn(InFile,Board[Left]);
      CLOSE(InFile);
      Left:=1;
END;

PROCEDURE Update(VAR Score: WORD; Pick: CHAR);
BEGIN
   IF Pick='L' THEN BEGIN
      INC(Score,Board[Left]);
      INC(Left);
   END ELSE BEGIN
      INC(Score,Board[Right]);
      DEC(Right);
   END;
END;

PROCEDURE WriteResult;
BEGIN
   ASSIGN(OutFile,'OUTPUT.TXT'); REWRITE(OutFile);
   WriteLn(OutFile,MyScore,' ',YourScore);
   CLOSE(OutFile);
END;

BEGIN
   Init;
   StartGame;

   REPEAT
      IF LeftScore(Left,Right)>=RightScore(Left,Right) THEN
         Move:='L'
      ELSE
         Move:='R';
      MyMove(Move);
      Update(MyScore,Move);

      YourMove(Move);
      Update(YourScore,Move);
   UNTIL Left>=Right;

   WriteResult;
END.

