PROGRAM Prefix;
(*-------------------------------------------------------------------------*)
(* Program, der finder lngste prefix. Metoden er beskrevet i PREFIX.TXT   *)
(*-------------------------------------------------------------------------*)
CONST                            (*----------------------------------------*)
  MaxN = 100;                    (* Maximal antal primitiver               *)
  MaxL = 20;                     (* Strst mulig lngde af primitiverne    *)
TYPE                             (*----------------------------------------*)
   STRL = STRING[MaxL];          (* Data type for primitiverne             *)
   STRB = STRING[2*MaxL];        (* Data type for bufferen                 *)
VAR                              (*----------------------------------------*)
   N: BYTE;                      (* Antal primitiver                       *)
   L: ARRAY[1..MaxN] OF BYTE;    (* lngden af primitiverne                *)
   P: ARRAY[1..MaxN] OF STRL;    (* Primitiverne                           *)
   PrimitiveNo ,                 (* Indeks for primitiv                    *)
   MaxLength   : BYTE;           (* Strste lngde af primitiv             *)
                                 (*----------------------------------------*)
   Buffer      : STRB;           (* Data buffer                            *)
   BufferLength,                 (* Aktuel lngde of bufferen              *)
   BufferIndex ,                 (* Index ved lsning til buffer           *)
   Place       : BYTE;           (* Position, hvorfra match forsges       *)
                                 (*----------------------------------------*)
   Offset      : BYTE;           (* Position i buffer ved lngste match    *)
   Cycle       ,                 (* Antal gennemlb - indlsninger af data *)
   Antal       : LONGINT;        (* lngden af lngste prefix              *)
                                 (*----------------------------------------*)
   MatchToHere : STRB;           (* Husker (..+..), hvortil der er match   *)
   NoMatch     : STRL;           (* Ingen match (--...-)                   *)
                                 (*----------------------------------------*)
   InFile      ,                 (* Input fil med primitiver               *)
   OutFile     ,                 (* Output fil med resultat                *)
   DataFile    : TEXT;           (* Data fil med sekvensen                 *)
                                 (*----------------------------------------*)

PROCEDURE Init;
BEGIN
   (* Indlser primitiver *)
      ASSIGN(InFile,'INPUT.TXT'); RESET(InFile);
      ReadLn(InFile,N); MaxLength:=0;
      FOR PrimitiveNo:=1 TO N DO BEGIN
         ReadLn(InFile,L[PrimitiveNo]); ReadLn(InFile,P[PrimitiveNo]);
         IF L[PrimitiveNo]>MaxLength THEN MaxLength:=L[PrimitiveNo];
      END;
      CLOSE(InFile);

   (* Klargr data *)
      ASSIGN(DataFile,'DATA.TXT'); RESET(DataFile);

      cycle:=0; Buffer:='';
      BufferLength:=2*MaxLength-1;
      Offset:=MaxLength+1;

      MatchToHere:='+';  (* Begynd med forsg p match fra start *)
      FOR BufferIndex:=1 TO BufferLength DO MatchToHere:=MatchToHere+'-';
      NoMatch:='';
      FOR BufferIndex:=1 TO MaxLength DO NoMatch:=NoMatch+'-';

      BufferIndex:=0;

END;

PROCEDURE Remember(nr: BYTE);
BEGIN
  MatchToHere[nr]:='+';
END;

PROCEDURE ReadBuffer;
BEGIN
   Buffer[0]:=CHR(BufferLength);  (* stter buffer lngden *)

   REPEAT (* Indls tegn *)
      INC(BufferIndex);
      ReadLn(DataFile,Buffer[BufferIndex]);
   UNTIL EOF(DataFile) OR (BufferIndex>=BufferLength);

   BufferIndex:=MaxLength-1;     (* Startindeks-1 ved n'ste indlsning *)
END;

PROCEDURE JusterBuffer;
VAR
   OldHere: StrB;
BEGIN
   (* Husk match stederne fr skift *)
      OldHere:=MatchToHere;
      DELETE(MatchToHere,1,MaxLength);

   IF MatchToHere=NoMatch THEN (* Ingen nye match *)
   (* Find sidste sted, hvor der var match *)
      REPEAT
         Dec(Offset);
      UNTIL OldHere[Offset]='+';

   MatchToHere:=MatchToHere+NoMatch;

   DELETE(Buffer,1,MaxLength);
END;


PROCEDURE TryMatchHere(Nr:BYTE);
VAR
  RestBuffer: STRB;
BEGIN
   RestBuffer:=Copy(Buffer,nr,MaxLength);
   FOR PrimitiveNo:=1 TO N DO
      IF (POS(P[PrimitiveNo],RestBuffer)=1) THEN Remember(Nr+L[PrimitiveNo]);
END;

PROCEDURE WriteResult;
BEGIN
   Antal:=(Cycle-1)*MaxLength+Offset-1;
   ASSIGN(OutFile,'OUTPUT.TXT'); REWRITE(OutFile);
   WriteLn(OutFile,Antal);
   CLOSE(OutFile);
END;

BEGIN
   Init;

   REPEAT
     INC(Cycle);
     ReadBuffer;
     FOR Place:=1 TO MaxLength DO IF MatchToHere[Place]='+' THEN TryMatchHere(Place);
     JusterBuffer;
   UNTIL Offset<=MaxLength;

   WriteResult;
END.

